import { RDFTerm } from "../AbstractWidget";
import { AutocompleteSparqlQueryBuilderIfc, ListSparqlQueryBuilderIfc } from "./SparqlBuilders";
import { SparqlFetcher } from "./UrlFetcher";
/**
 * Interface for objects that can provide data to a ListWidget :
 * either through a SPARQL query, or through custom mean (calling an API)
 */
export interface ListDataProviderIfc {
    getListContent(domain: string, predicate: string, range: string, lang: string, typePredicate: string, callback: (items: {
        term: RDFTerm;
        label: string;
    }[]) => void): void;
}
/**
 * Implementation of ListDataProviderIfc that executes a SPARQL query against an endpoint,
 * and read the 'uri' and 'label' columns.
 */
export declare class SparqlListDataProvider implements ListDataProviderIfc {
    queryBuilder: ListSparqlQueryBuilderIfc;
    sparqlFetcher: SparqlFetcher;
    constructor(sparqlEndpointUrl: any, queryBuilder: ListSparqlQueryBuilderIfc);
    getListContent(domainType: string, predicate: string, rangeType: string, lang: string, typePredicate: string, callback: (items: {
        term: RDFTerm;
        label: string;
    }[]) => void): void;
    getRdfTermColumn(aBindingSet: any): string | undefined;
}
/**
 * Interface for objects that can provide data to a LiteralListWidget :
 * either through a SPARQL query, or through custom mean (calling an API)
 */
export interface LiteralListDataProviderIfc {
    getListContent(domain: string, predicate: string, range: string, lang: string, typePredicate: string, callback: (values: {
        literal: string;
    }[]) => void): void;
}
/**
 * Implementation of LiteralListDataProviderIfc that executes a SPARQL query against an endpoint,
 * and read the 'uri' and 'label' columns.
 */
export declare class SparqlLiteralListDataProvider implements LiteralListDataProviderIfc {
    queryBuilder: ListSparqlQueryBuilderIfc;
    sparqlFetcher: SparqlFetcher;
    constructor(sparqlEndpointUrl: any, queryBuilder: ListSparqlQueryBuilderIfc);
    getListContent(domainType: string, predicate: string, rangeType: string, lang: string, typePredicate: string, callback: (values: {
        literal: string;
    }[]) => void): void;
}
/**
 * Implementation of ListDataProviderIfc that wraps a LiteralListDataProviderIfc so that the same ListWidget
 * can be used with LiteralList
 */
export declare class ListDataProviderFromLiteralListAdpater implements ListDataProviderIfc {
    literalListDataProvider: LiteralListDataProviderIfc;
    constructor(literalListDataProvider: LiteralListDataProviderIfc);
    getListContent(domainType: string, predicate: string, rangeType: string, lang: string, typePredicate: string, callback: (items: {
        term: RDFTerm;
        label: string;
    }[]) => void): void;
}
/**
 * Interface for objects that can provide data to an AutocompleteWidget :
 * either through a SPARQL query, or through custom mean (calling an API)
 */
export interface AutocompleteDataProviderIfc {
    autocompleteUrl(domain: string, predicate: string, range: string, key: string, lang: string, typePredicate: string): string;
    listLocation(data: any): any;
    elementLabel(element: any): string;
    elementRdfTerm(element: any): any;
}
/**
 * Implementation of AutocompleteDataProviderIfc that executes a SPARQL query against an endpoint,
 * and read the 'uri' and 'label' columns.
 */
export declare class SparqlAutocompleDataProvider implements AutocompleteDataProviderIfc {
    queryBuilder: AutocompleteSparqlQueryBuilderIfc;
    sparqlFetcher: SparqlFetcher;
    constructor(sparqlEndpointUrl: any, queryBuilder: AutocompleteSparqlQueryBuilderIfc);
    autocompleteUrl(domain: string, predicate: string, range: string, key: string, lang: string, typePredicate: string): string;
    listLocation(data: any): any;
    elementLabel(element: any): string;
    elementRdfTerm(element: any): any;
}
